%bcond_with shared_libuv
%bcond_with shared_cares
%bcond_with shared_brotli
%bcond_with shared_icu

%define major 16
%define minor 20
%define fix 2

%define node_prefix %{_libdir}/%{name}

Name: nodejs%{major}
Version: %{major}.%{minor}.%{fix}
Release: 1
License: MIT
Summary: Node.js JavaScript Runtime
Url: https://www.nodejs.org

BuildRequires: pkg-config
BuildRequires: python3
BuildRequires: zlib-devel
BuildRequires: openssl-1.1-devel
%{?with_shared_icu:BuildRequires: icu-devel}
%{?with_shared_cares:BuildRequires: c-ares-devel}
%{?with_shared_libuv:BuildRequires: libuv-devel}
%{?with_shared_brotli:BuildRequires: brotli-devel}

BuildRequires: gcc10
BuildRequires: gcc10-cplusplus
BuildRequires: libstdcplusplus6-10-devel >= 10.2.0

Requires: libopenssl1_1 >= 1.1.1-6
Requires: python3
Requires: update-alternatives

Requires: libstdcplusplus6 >= 10.2.0
Requires: libgcc_s1 >= 10.2.0

Requires(post): update-alternatives
Requires(postun): update-alternatives

Conflicts: nodever < 1.0.0

Source0: https://nodejs.org/dist/v%{version}/node-v%{version}.tar.gz

Patch0: nodejs16-server-hang-TCP-RST.patch

%description
Node.js is a JavaScript runtime built on Chrome V8 JavaScript engine. Node.js
uses an event-driven, non-blocking I/O model that makes it lightweight and
efficient.

%prep
%setup -q -n node-v%{version}

%patch0 -p1

# Fix the wrong dependency /usr/bin/python
find deps tools ./configure* -type f | xargs perl -pi -e 's|/usr/bin/python|%{_bindir}/python3|g'

%build

export CC=gcc-10
export CXX=g++-10

./configure \
  --prefix=%{node_prefix} \
  --dest-cpu=ppc64 \
  --without-node-snapshot \
  --shared-openssl \
  --shared-zlib \
   %{?with_shared_icu:--with-intl=system-icu} \
   %{?with_shared_cares:--shared-cares} \
   %{?with_shared_libuv:--shared-libuv} \
   %{?with_shared_brotli:--shared-brotli} \
  # end

%make_build
# basic verification
./node -e "console.log(process.versions)"

%check

make test-only

%install

%make_install V=1 PORTABLE=1

# Create an "npm builtin config file" to set the shell used to bash
# If users want to override, they can do so using the per-user
# or per-project config files:
# https://docs.npmjs.com/cli/v8/configuring-npm/npmrc#files
cd %{buildroot}%{node_prefix}

mkdir ./etc
cat <<EOF > ./etc/npmrc
script-shell=%{_bindir}/bash
EOF

%post
###############################################################################
# processing after the install
# $1 = 1: install
# $1 = 2+: upgrade
###############################################################################

type=upgrade
[ $1 -eq 1 ] && type=install

if [ $type = 'install' ]
then
  # set up the just installed version of Node.js in update-alternatives
  %{_bindir}/alternatives --install %{_bindir}/node node %{_libdir}/nodejs%{major}/bin/node %{major} \
  --slave %{_bindir}/npm npm %{_libdir}/nodejs%{major}/bin/npm \
  --slave %{_bindir}/npx npx %{_libdir}/nodejs%{major}/bin/npx
fi


%postun
###############################################################################
# processing after the uninstall
# $1 = 0: uninstall
# $1 = 1: upgrade
###############################################################################

type=uninstall
[ $1 -eq 1 ] && type=upgrade

# After uninstallation, need to check if it was truly an uninstallation, and if
# so, remove the the update-alternatives entry for this version of node
if [ $type = 'uninstall' ]
then
  %{_bindir}/alternatives --remove node %{_libdir}/nodejs%{major}/bin/node
fi


%files
# Set the default owner to QSYS and group to *NONE
%defattr(-, qsys, *none)

%{node_prefix}

%changelog
* Wed Aug 9 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 16.20.2
- Update for security release
- https://github.com/nodejs/node/releases/tag/v16.20.2A
- https://nodejs.org/en/blog/vulnerability/august-2023-security-releases

* Tue Jun 20 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 16.20.1
- Update for security release
- https://nodejs.org/en/blog/vulnerability/june-2023-security-releases

* Fri Feb 17 2023 Xu Meng <mengxumx@cn.ibm.com> - 16.19.1
- update to address below vulnerabilities -
- CVE-2023-23918
- CVE-2023-23919
- CVE-2023-23936
- CVE-2023-24807
- CVE-2023-23920

* Thu Feb 09 2023 Kevin Adler <kadler@us.ibm.com> - 16.18.1
- BuildRequire openssl-1.1-devel instead of openssl-devel

* Thu Jan 05 2023 Kevin Adler <kadler@us.ibm.com> - 16.18.1
- Create npmrc inside node_prefix not build directory

* Wed Nov 9 2022 Xu Meng <mengxumx@cn.ibm.com> - 16.18.1
- update to address one vulnerability -
- CVE-2022-43548

* Wed Sep 28 2022 Xu Meng <mengxumx@cn.ibm.com> - 16.17.1
- update to address below vulnerabilities -
- CVE-2022-32212
- CVE-2022-32215
- CVE-2022-32213
- CVE-2022-35256
- CVE-2022-32222
- CVE-2022-35255

* Thu Aug 25 2022 Xu Meng <mengxumx@cn.ibm.com> - 16.16.0
- set proper script shell for running npm

* Fri Jul 8 2022 Xu Meng <mengxumx@cn.ibm.com> - 16.16.0
- update to address below vulnerabilities -
- CVE-2022-32213
- CVE-2022-32214
- CVE-2022-32215
- CVE-2022-32212

* Mon Mar 21 2022 Xu Meng <mengxumx@cn.ibm.com> - 16.14.2
- update to address a vulnerability -
- CVE-2022-0778

* Wed Feb 16 2022 Xu Meng <mengxumx@cn.ibm.com> - 16.13.2
- Handle server hang when remote sends TCP RST

* Tue Jan 11 2022 Xu Meng <mengxumx@cn.ibm.com> - 16.13.2
- update to address four vulnerabilities -
- CVE-2021-44531
- CVE-2021-44532
- CVE-2021-44533
- CVE-2022-21824

* Wed Oct 13 2021 Xu Meng <mengxumx@cn.ibm.com> - 16.11.1
- update to address two vulnerabilities -
- CVE-2021-22959
- CVE-2021-22960

* Fri Aug 13 2021 Xu Meng <mengxumx@cn.ibm.com> - 16.6.2
- update to 16.6.2 to address three vulnerabilities -
- CVE-2021-22931
- CVE-2021-22940
- CVE-2021-22939

* Thu Aug 5 2021 Xu Meng <mengxumx@cn.ibm.com> - 16.6.0
- update to v16.6.0 to address vulnerability -
- CVE-2021-22930

* Fri Jul 2 2021 Xu Meng <mengxumx@cn.ibm.com> - 16.4.1
- update to v16.4.1 to address a vulnerability -
- CVE-2021-22918

* Mon May 10 2021 Xu Meng <mengxumx@cn.ibm.com> - 16.1.0
- Update to v16.1.0

* Wed Apr 21 2021 Xu Meng <mengxumx@cn.ibm.com> - 16.0.0
- Fix crash on systems with VLAN interface names > 10 characters

* Wed Apr 21 2021 Xu Meng <mengxumx@cn.ibm.com> - 16.0.0
- Update to v16.0.0

* Thu Apr 8 2021 Xu Meng <mengxumx@cn.ibm.com> - 15.14.0
- Update to v15.14.0

* Wed Mar 3 2021 Xu Meng <mengxumx@cn.ibm.com>
- Initial package for verification
